#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "struct_patient.h"
#include "file_opr.c"

#define MAX_SIZE 20
#define WARNING "malloc failed!"

Patient_Node* create_new_patient();
void insert_patient_to_list(Patient_Node *new_node, Patient_Node **head);
void delete_patient(char *name, Patient_Node **head);


Patient_Node* create_new_patient(){
	Patient_Node *new_node;
	// Allocation memory for new patient
	new_node = (Patient_Node*)malloc(sizeof(Patient_Node));
	
	if(new_node == NULL)
	{
		printf(WARNING);
		exit(1);
	}
	
	// Getting inputs from user
	printf("\nPlease enter patient informations..\n");
	printf("Name   :");
	scanf(" %s", new_node->data.name);
	printf("Surname:");
	scanf(" %s", new_node->data.surname);
	printf("ID     :");
	scanf(" %s", new_node->data.ID);
	printf("Age    :");
	scanf("%d", &new_node->data.age);
	printf("If patient has a priorty enter 1 otherwise enter 0 for prior..\n");
	printf("Prior  :");
	scanf("%d", &new_node->data.prior);
	new_node->next = NULL;
	return new_node;
}

// This function is used for inserting new patient to list.
void insert_patient_to_list(Patient_Node *new_node, Patient_Node **head){
	// If head equals to NULL, new patient will be the head of list.
	if((*head) == NULL)
	{
		(*head) = new_node;
		return;
	}
	
	Patient_Node *var_head = (*head);
	Patient_Node *last_prior = (*head);
	Patient_Node *temp;
	// Check the new patient priorty, Has it high priority.
	if(new_node->data.prior != 0)
	{
		// Detect the last patient with high priorty.
		while(var_head != NULL)
		{
			if(var_head->data.prior != 0)
				last_prior = var_head;
			var_head = var_head->next;
		}
		
		// We set the initial last prior as head node.
		// Check is the last prior still head of the list.
		if(last_prior == (*head) )
		{
			// If head of list has not high priorty, new patient will be head of list.
			if((*head)->data.prior == 0)
			{
				temp = (*head);
				(*head) = new_node;
				new_node->next = temp;
			}
			else // else, new patient will be added after the last prior.
			{
				temp = last_prior->next;
				last_prior->next = new_node;
				new_node->next = temp;
			}
		}
		else // else, new patient will be added after the last prior.
		{
			temp = last_prior->next;
			last_prior->next = new_node;
			new_node->next = temp;
		}
	}
	else // If new patient has low priorty, add new patient at the end of the list.
	{
		while(var_head->next != NULL)
		{
			var_head = var_head->next;
		}
		var_head->next = new_node;
	}
}

// This function is used for deleting patient with name.
void delete_patient(char *name, Patient_Node **head){
	Patient_Node *patient = NULL, *var_head;
	var_head = (*head);
	while(var_head != NULL)
	{
		// If patient's name and entered name is equal, variable patient keeps patient that will delete.
		if(strcmp(var_head->data.name, name) == 0) // 0 means, two string is equal.
			patient = var_head;
			
		var_head = var_head->next;
	}
	
	// If patient is the head of list, new head will be the head->next.
	if(patient == (*head))
		(*head) = (*head)->next;
	else
	{
		// If patient is NULL, there is no patient in list with entered name.
		if(patient == NULL)
		{
			printf("\nCould not find the element with %s", name);
			return;	
		}
		var_head = (*head);
		// Finding patient in list.
		while(var_head->next != patient)
		{
			var_head = var_head->next;
		}
		// When we delete the patient, previos node's next node will be the patient's next node.
		var_head->next = var_head->next->next;
		free(patient);
	}
}


int main() {
	char name[MAX_SIZE];
	int choose = 1;
	// head node specify the head of list.
	Patient_Node *head = NULL;
	while(choose > 0)
	{
		printf("\nFor entering new patient press 1..\n");
		printf("For deleting patient with name press 2..\n");
		printf("For reading list from the file press 3..\n");
		printf("For exit press 0..\n");
		printf("\nPlease enter your choice:");
		scanf(" %d", &choose);
		switch(choose){
			case 1:
				insert_patient_to_list(create_new_patient(), &head);
				printListWithAllInformations(&head);	
				break;
			case 2:
				printf("\nEnter patient name that want to delete : ");
				scanf(" %s", name);
				delete_patient(name, &head);
				printListWithAllInformations(&head);
				break;
			case 3:
				read_File();
				break;
		}
	}	
	return 0;
}
